
// Analog Measurement
volatile int SampleTotal=750;
volatile int ADCSampleCount=0;                      // counts samples in AnalogValue
volatile int ADCState;                              // state counter for sampling 1: sample, 2: finish. 3=4 samples taken 
volatile int SampleCount;
volatile uint8_t AnalogPin;                     
volatile int AnalogVal[750];
int ADCNo=1;
float Max1;
float Max2;
float Max3;
float Max4;
float Min1;
float Min2;
float Min3;
float Min4;

// frequency measurement
const byte        interruptPin = 25;              // Assign the interrupt pin
volatile uint64_t StartValue;                     // First interrupt value
volatile uint64_t PeriodCount;                    // period in counts of 0.000001 of a second
float             Freg;                           // frequency
int               RPM;                            // 4 cyclinder RPM - Freq x 120     

 // timer setups          
hw_timer_t * timer0 = NULL;                   // timer0 used for analogs pointer to a variable of type hw_timer_t
hw_timer_t * timer1 = NULL;                   // timer1 used for digital/freq measurement pointer to a variable of type hw_timer_t 
portMUX_TYPE Freqmux = portMUX_INITIALIZER_UNLOCKED;   // allows for synchronising between the ISR and the main code 
portMUX_TYPE mux = portMUX_INITIALIZER_UNLOCKED;  

//-------------------------------------------------------------
// Digital Event Interrupt
// This event triggers on the falling value on the interrupt pin
// ISR located in IRAM
//-------------------------------------------------------------
void IRAM_ATTR handleInterrupt() 
{
   portENTER_CRITICAL_ISR(&Freqmux);
    uint64_t TempVal= timerRead(timer1);        // value of timer at interrupt
    PeriodCount= TempVal - StartValue;          // period count between rising edges in 0.000001 of a second
    StartValue = TempVal;                       // puts latest reading as start for next calculation
    portEXIT_CRITICAL_ISR(&Freqmux);
}

//----------------------------------------------------------
// ADC Interrupt
// This interrupt is triggered by the timer0 and samples the
// analogs until the total sample is exceeded
// ISR located in IRAM
//------------------------------------------------------------

void IRAM_ATTR onTimer() 
 {
   portENTER_CRITICAL_ISR(&mux);
   if ((ADCSampleCount<SampleTotal)&&(ADCState==1))        
      {
       AnalogVal[ADCSampleCount]=analogRead(AnalogPin);        
       ADCSampleCount++;
       SampleCount++;
      }
     if (ADCSampleCount>=SampleTotal)
      {
       ADCSampleCount=0;                               // sets for next sample
       ADCState = 2;                                   // signals completion to main program
      }
   portEXIT_CRITICAL_ISR(&mux);
 } // end ISR


//----------------------------------------------------------------
// Setup
//----------------------------------------------------------------
void setup() 
{ 

 Serial.begin(115200);                    // Debug
 pinMode(interruptPin, INPUT_PULLUP);             // sets pin high
 
 // analog stuff
 analogSetWidth(12);
 analogSetAttenuation(ADC_11db);
 analogReadResolution(12);                       //0-4095
 ADCNo=1;                                        // Analog input counter                                
 AnalogPin=34;                                   // start pin for ADCNO 1
 ADCState=1;                                     //1 = sampling 2 = completed 

// Analog Interrupt SetUp
 timer0 = timerBegin(0, 80, true);               // this returns a pointer to the hw_timer_t global variable
                                                 // 0 = first timer (analog)
                                                 // 80 is prescaler so 80MHZ divided by 80 = 1MHZ signal ie 0.000001 of a second
                                                 // true - counts up
  timerAttachInterrupt(timer0, &onTimer, true);  // attaches the timer to a pointer to the interrupt routine onTimer, true means rising edge
  timerAlarmWrite(timer0, 300, true);            // this attaches the timer to the interrupt every 300 counts (0.0003 second = 3/10mS)
                                                 // thus 750 counts = 0.225 sec
                                                 // true means the counter will reset after each interrupt
  timerAlarmEnable(timer0);  
  delay(200);
                                               
 // frequency interrupt setup
  attachInterrupt(digitalPinToInterrupt(interruptPin), handleInterrupt, FALLING); // attaches pin to interrupt on Falling Edge
  timer1 = timerBegin(1, 80, true);                                               // this returns a pointer to the hw_timer_t global variable
                                                                                  // 1 = first timer(freq)
                                                                                  // 80 is prescaler so 80MHZ divided by 80 = 1MHZ signal ie 0.000001 of a second
                                                                                  // true - counts up
  timerStart(timer1);                                                             // Start the timer
  delay(200);   
  
  Serial.println(" Setup Complete");
}    // setup


//-----------------------------------------------
//Calc RPM from frequency interrupt
//----------------------------------------------

 void CalcRPM()
 {
  portENTER_CRITICAL(&Freqmux);
  Freg =1000000.00/PeriodCount;                       // PeriodCount in 0.000001 of a second
  portEXIT_CRITICAL(&Freqmux);
  Serial.print("Frequency   ");Serial.println(Freg,2);
  Serial.println(millis());  
  //delay(200);
 }

//-----------------------------------------------------------------
// Main Program
//-----------------------------------------------------------------
void loop() 
{
// Calculates values at end of analog sample  
//--------------------------------------------------------- 
 if (ADCState==2)                     // Samplng completed
   {   
   AnalogMaxMin(ADCNo);                // Finds the Max and Min values
   CalcRPM();                          // Calculates the RPM


// increments the analog count
//-----------------------------------------------------------------
    ADCNo++;                              
    if (ADCNo>4) ADCNo=1;              // stops above 4 count  

// set up new analog
//---------------------------------------------------------------
  portENTER_CRITICAL_ISR(&mux);
   SampleCount=0;
    switch (ADCNo)
      {
      case 1: AnalogPin = 34; break;    // change analog
      case 2: AnalogPin = 35; break;
      case 3: AnalogPin = 32; break;
      case 4: AnalogPin = 33; break;
      }  
 portEXIT_CRITICAL_ISR(&mux); 
     ADCState=1;                        //reset sampling 
   }  // sample completed  


} // end loop
